#!/usr/bin/env python

"A form submission test."

import WebStack.Generic

class FormResource:

    """
    A resource which handles incoming form submissions.
    """

    def respond(self, trans):

        """
        Examine the incoming request, decoding the form information.
        """

        # NOTE: Some frameworks do not pass in the content type.

        content_type = trans.get_content_type()
        if content_type:
            content_type_str = content_type.media_type
            content_type_charset = content_type.charset
        else:
            content_type_str = None
            content_type_charset = None

        # Optional encodings can be employed.

        fields_from_path = trans.get_fields_from_path()

        # Send the appropriate kind of response.

        if fields_from_path.has_key("charset"):
            charset = fields_from_path["charset"][0]
            trans.set_content_type(WebStack.Generic.ContentType("text/html", charset))
        elif content_type_charset:
            charset = content_type_charset
            trans.set_content_type(WebStack.Generic.ContentType("text/html", charset))
        else:
            charset = None
            trans.set_content_type(WebStack.Generic.ContentType("text/html"))

        # Handle charset issues.

        if charset:
            fields = trans.get_fields_from_body(charset)
        else:
            fields = trans.get_fields_from_body()

        out = trans.get_response_stream()

        # Use Unicode for correct character encoding behaviour.

        out.write(u"""
<html>
  <head>
    <title>Form Test</title>
  </head>
  <body>
    <h1>Form Test</h1>
    <h2>Normal</h2>
    <form method="post" action="">
      <input name="x" type="text" value="1"/><br/>
      <input name="x" type="text" value="2"/><br/>
      <input name="y" type="text" value="3"/><br/>
      <input name="f" type="file"/><br/>
      <input name="send" type="submit" value="Send!"/>
    </form>
    <h2>Multipart</h2>
    <form method="post" action="" enctype="multipart/form-data">
      <input name="x" type="text" value="1"/><br/>
      <input name="x" type="text" value="2"/><br/>
      <input name="y" type="text" value="3"/><br/>
      <input name="f" type="file"/><br/>
      <input name="send" type="submit" value="Send!"/>
    </form>
    <h2>Content Type</h2>
    <p>%s</p>
    <h2>Charset</h2>
    <h3>From Content Type</h3>
    <p>%s</p>
    <h3>In Use</h3>
    <p>%s</p>
    <h2>Fields from Body</h2>
    <ul>%s</ul>
  </body>
</html>
""" % (
            content_type_str,
            content_type_charset,
            charset,
            self._format_fields(fields)
        ))

    def _format_fields(self, d):
        return "".join([
            "<li>%s<ul>%s</ul></li>" % (key, self._format_list(value))
            for key, value in d.items()
        ])

    def _format_list(self, l):
        return "".join([
            "<li>%s</li>" % value
            for value in l
        ])

# vim: tabstop=4 expandtab shiftwidth=4
